@@ -1,4 +1,4 @@
-# TBlock - An anticapitalist ad-blocker that uses the hosts file
+# TBlock - An anti-capitalist ad-blocker that uses the hosts file
 # Copyright (C) 2021 Twann <twann@ctemplar.com>
 
 # This program is free software: you can redistribute it and/or modify
@@ -15,9 +15,9 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 __license__ = 'GPLv3'
-__version__ = '1.1.4'
+__version__ = '1.3.1'
 
-run_help = '''TBlock - An anticapitalist ad-blocker that uses the hosts file
+run_help = '''TBlock - An anti-capitalist ad-blocker that uses the hosts file
 Copyright (c) 2021 Twann <twann@ctemplar.com>
 
 usage: tblock <operation> <options>
@@ -47,6 +47,7 @@ usage: tblock <operation> <options>
 * Hosts
   -H  --update-hosts                      Update hosts file
   -D  --restore                           Restore default hosts file
+  -G  --gen-hosts                         Generate a new hosts file based on hostname
 
 <options>
 * General options
