@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# TBlock - An anticapitalist ad-blocker that uses the hosts file
+# TBlock - An anti-capitalist ad-blocker that uses the hosts file
 # Copyright (C) 2021 Twann <twann@ctemplar.com>
 
 # This program is free software: you can redistribute it and/or modify
@@ -28,7 +28,7 @@ import requests
 import urllib3
 
 # Local libraries
-from .rules import get_all_rules
+from .rules import get_all_rules, get_wildcards_rules
 from .converter.parser import FilterParser
 from .config import Path, Font
 from .utils import check_root, prompt_user, is_url, download_file, lock_database, unlock_database
@@ -94,7 +94,7 @@ class Filter:
         # Fetch filter data from the database
         with sqlite3.connect(Path.DATABASE) as conn:
             data = conn.cursor().execute(
-                'SELECT "source", "metadata", "subscribing", "on_rfr", "permissions" FROM "filters" WHERE id=?;',
+                'SELECT "source", "metadata", "subscribing", "on_rfr", "permissions", "mirrors" FROM "filters" WHERE id=?;',
                 (self.id,)
             ).fetchall()
         self.exists = bool(data)
@@ -103,6 +103,8 @@ class Filter:
         self.subscribing = bool(data[0][2]) if data else False
         self.on_rfr = bool(data[0][3]) if data else False
         self.permissions = FilterPermissions(data[0][4]) if data else None
+        self.mirrors = json.loads(data[0][5]) if data and data[0][5] else []
+        self.tmp_filter = None
 
     def subscribe(self, permissions: FilterPermissions, source: str = None) -> None:
         """Mark a filter as subscribed in the database
@@ -236,6 +238,32 @@ class Filter:
             )
         self.update(False)
 
+    def __get_mirror(self, mirror) -> bool:
+        print(f' [{Fore.YELLOW}|{Style.RESET_ALL}] Get: {mirror}', end='\r')
+        if is_url(mirror):
+            self.tmp_filter = os.path.join(Path.TMP_DIR, self.id + '.tmp')
+            try:
+                download_file(mirror, self.tmp_filter)
+            except (requests.exceptions.ConnectionError, urllib3.exceptions.MaxRetryError, ConnectionRefusedError,
+                    urllib3.exceptions.NewConnectionError, InvalidLinkError):
+                print(f' [{Fore.RED}x{Style.RESET_ALL}] Get: {mirror}')
+                return False
+            else:
+                print(f' [{Fore.GREEN}{Font.CHECK_MARK}{Style.RESET_ALL}] Get: {mirror}')
+                return True
+        else:
+            # This sequence is just meant to show an error message in the terminal before exiting
+            self.tmp_filter = mirror
+            try:
+                with open(self.tmp_filter, 'rt') as f:
+                    f.close()
+            except FileNotFoundError:
+                print(f' [{Fore.RED}x{Style.RESET_ALL}] Get: {mirror}')
+                return False
+            else:
+                print(f' [{Fore.GREEN}{Font.CHECK_MARK}{Style.RESET_ALL}] Get: {mirror}')
+                return True
+
     def update(self, verbosity: bool = True, subscribing_operation: bool = False) -> None:
         """Update rules from a filter
 
@@ -253,31 +281,40 @@ class Filter:
         # Download the filter is it is online
         print(f' [{Fore.YELLOW}|{Style.RESET_ALL}] Get: {self.source}', end='\r')
         if is_url(self.source):
-            tmp_filter = os.path.join(Path.TMP_DIR, self.id + '.tmp')
+            self.tmp_filter = os.path.join(Path.TMP_DIR, self.id + '.tmp')
             try:
-                download_file(self.source, tmp_filter)
+                download_file(self.source, self.tmp_filter)
             except (requests.exceptions.ConnectionError, urllib3.exceptions.MaxRetryError, ConnectionRefusedError,
                     urllib3.exceptions.NewConnectionError, InvalidLinkError):
                 print(f' [{Fore.RED}x{Style.RESET_ALL}] Get: {self.source}')
-                raise NetworkError(f'{Fore.RED}ERROR: cannot download: {self.source}{Style.RESET_ALL}')
+                for mirror in self.mirrors:
+                    if self.__get_mirror(mirror):
+                        break
+                else:
+                    raise NetworkError(f'{Fore.RED}ERROR: failed to retrieve filter: {self.id}{Style.RESET_ALL}')
+            else:
+                print(f' [{Fore.GREEN}{Font.CHECK_MARK}{Style.RESET_ALL}] Get: {self.source}')
         else:
             # This sequence is just meant to show an error message in the terminal before exiting
-            tmp_filter = self.source
+            self.tmp_filter = self.source
             try:
-                with open(tmp_filter, 'rt') as f:
+                with open(self.tmp_filter, 'rt') as f:
                     f.close()
             except FileNotFoundError:
                 print(f' [{Fore.RED}x{Style.RESET_ALL}] Get: {self.source}')
-                raise FileNotFoundError(f'cannot find file: {self.source}')
+                for mirror in self.mirrors:
+                    if self.__get_mirror(mirror):
+                        break
+                else:
+                    raise FileNotFoundError(f'ERROR: failed to retrieve filter: {self.id}')
             else:
-                print(f' [{Fore.YELLOW}|{Style.RESET_ALL}] Get: {self.source}', end='\r')
-        print(f' [{Fore.GREEN}{Font.CHECK_MARK}{Style.RESET_ALL}] Get: {self.source}')
+                print(f' [{Fore.GREEN}{Font.CHECK_MARK}{Style.RESET_ALL}] Get: {self.source}')
 
         # Check if the filter has a specified syntax
         try:
-            rules = FilterParser(tmp_filter, syntax=self.metadata['syntax'])
+            rules = FilterParser(self.tmp_filter, syntax=self.metadata['syntax'])
         except KeyError:
-            rules = FilterParser(tmp_filter)
+            rules = FilterParser(self.tmp_filter)
 
         self.delete_all_rules()
 
@@ -285,11 +322,12 @@ class Filter:
         all_rules = rules.get_filter_content(
             self.permissions.allow, self.permissions.block, self.permissions.redirect, False
         )
+        all_wildcards_rules = get_wildcards_rules()
         all_rules_in_db = get_all_rules()
 
         # Remove the filter only if it has been downloaded in a temporary location
-        if self.source != tmp_filter:
-            os.remove(tmp_filter)
+        if self.source != self.tmp_filter and self.tmp_filter not in self.mirrors:
+            os.remove(self.tmp_filter)
 
         count = 1
         all_rules_count = len(all_rules)
@@ -323,6 +361,12 @@ class Filter:
 
                 elif rule[2] == 'allow' and self.permissions.allow or rule[2] == 'block' and self.permissions.block:
 
+                    # Support for wildcard rules, this can be slow (depending of the number of wildcard rules)
+                    if rule[2] == 'block':
+                        for wildcard in all_wildcards_rules:
+                            if re.match(wildcard.replace('*', '.*'), rule[1]):
+                                continue
+
                     # Avoid allowing or redirecting rules to be overwritten by blocking rules
                     # See: https://codeberg.org/tblock/tblock/issues/4
                     if rule[1] in all_rules_in_db.keys():
@@ -346,6 +390,11 @@ class Filter:
 
                 elif rule[2] == 'redirect' and self.permissions.redirect:
 
+                    # Support for wildcard rules, this can be slow (depending of the number of wildcard rules)
+                    for wildcard in all_wildcards_rules:
+                        if re.match(wildcard.replace('*', '.*'), rule[1]):
+                            continue
+
                     # Avoid allowing rules to be overwritten by redirecting rules
                     # See: https://codeberg.org/tblock/tblock/issues/4
                     if rule[1] in all_rules_in_db.keys():
