@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# TBlock - An anticapitalist ad-blocker that uses the hosts file
+# TBlock - An anti-capitalist ad-blocker that uses the hosts file
 # Copyright (C) 2021 Twann <twann@ctemplar.com>
 
 # This program is free software: you can redistribute it and/or modify
@@ -17,16 +17,17 @@
 
 # Standard libraries
 import sqlite3
+import re
 
 # External libraries
 from colorama import Fore, Style
 
 # Local libraries
 from .hosts import update_hosts
-from .config import Path, Font
-from .utils import is_valid_ip, extract_domain, check_root, prompt_user, lock_database, unlock_database
+from .config import is_valid_ip, Path, Font, Var
+from .utils import extract_domain, check_root, prompt_user, lock_database, unlock_database, is_valid_domain
 from .exceptions import InvalidBindAddress, RuleExistsError, InvalidRulePolicy, \
-    RuleNotExistsError, MissingArgumentError, SIGTERM
+    RuleNotExistsError, MissingArgumentError, SIGTERM, InvalidDomainError
 
 
 class Rule:
@@ -66,12 +67,25 @@ class Rule:
         elif policy == 'redirect' and not ip:
             print(f' [{Fore.RED}x{Style.RESET_ALL}] Adding rule for {self.domain}')
             raise InvalidBindAddress('redirecting IP address cannot be NoneType')
-        elif policy == 'redirect' and not is_valid_ip(ip):
+        elif policy == 'redirect' and not is_valid_ip(ip, Var.ALLOW_IPV6):
             print(f' [{Fore.RED}x{Style.RESET_ALL}] Adding rule for {self.domain}')
-            raise InvalidBindAddress('redirecting IP address cannot be NoneType')
+            raise InvalidBindAddress('redirecting IP address seems invalid')
         elif policy == self.policy and ip == self.ip and self.priority:
-            print(f' [{Fore.GREEN}{Font.CHECK_MARK}{Style.RESET_ALL}] Adding rule for {self.domain}')
+            print(f' [{Fore.RED}x{Style.RESET_ALL}] Adding rule for {self.domain}')
             raise RuleExistsError(f'exact same rule already exists for domain: "{self.domain}"')
+        elif not policy == 'allow' and not is_valid_domain(self.domain):
+            print(f' [{Fore.RED}x{Style.RESET_ALL}] Adding rule for {self.domain}')
+            raise InvalidDomainError(f'invalid domain: {self.domain}')
+        elif policy == 'allow' and re.match(re.compile(r'[$/&%^?:()=,;#\"+\t_\\]'), self.domain):
+            print(f' [{Fore.RED}x{Style.RESET_ALL}] Adding rule for {self.domain}')
+            raise InvalidDomainError(f'invalid domain: {self.domain}')
+
+        if policy == 'block' or policy == 'redirect':
+            # Support for wildcard rules, this can be slow (depending of the number of wildcard rules)
+            for wildcard in get_wildcards_rules():
+                if re.match(wildcard.replace('*', '.*'), self.domain):
+                    print(f' [{Fore.RED}x{Style.RESET_ALL}] Adding rule for {self.domain}')
+                    raise RuleExistsError(f'wildcard rule already exists for domain: "{self.domain}"')
 
         with sqlite3.connect(Path.DATABASE) as db:
             if self.exists:
@@ -140,6 +154,23 @@ def get_all_rules(standard: bool = True, user: bool = True) -> dict:
     return all_rules
 
 
+def get_wildcards_rules() -> list:
+    """Get all wildcards rules
+
+    Returns:
+        list: All rules in a list
+    """
+    all_rules = []
+    with sqlite3.connect(Path.DATABASE) as db:
+        data = db.cursor().execute(
+            'SELECT "domain" FROM "rules" WHERE policy="allow";'
+        ).fetchall()
+    for rule in data:
+        if re.findall(r'\*', rule[0]):
+            all_rules.append(rule[0])
+    return all_rules
+
+
 def add_rules(rules: list, force: bool = False) -> None:
     """Add rules for domains
 
@@ -171,6 +202,8 @@ def add_rules(rules: list, force: bool = False) -> None:
             print(f'{Fore.YELLOW}WARNING: {err}{Style.RESET_ALL}')
         except InvalidBindAddress as err:
             raise InvalidBindAddress(f'{Fore.RED}ERROR: {err}{Style.RESET_ALL}')
+        except InvalidDomainError as err:
+            raise InvalidDomainError(f'{Fore.RED}ERROR: {err}{Style.RESET_ALL}')
     update_hosts(get_all_rules())
     unlock_database()
 
@@ -191,7 +224,7 @@ def delete_rules(rules: list, force: bool = False) -> None:
         raise MissingArgumentError(
             f'{Fore.RED}ERROR: you need to specify at least one domain to delete from rules{Style.RESET_ALL}'
         )
-    elif not force and not prompt_user('You are about to add rules for the following domains:', list_domains):
+    elif not force and not prompt_user('You are about to delete rules for the following domains:', list_domains):
         raise SIGTERM()
     print(':: Deleting rules...')
     lock_database()
